import QtQml.Models 2.3
import QtQuick 2.9
import QtQuick.Controls 2.2
import QtQuick.Layouts 1.3
import RustCode 1.0;

ApplicationWindow {
    visible: true
    width: 450
    height: 580
    header: ToolBar {
        Label {
            anchors.fill: parent
            text: qsTr("todos")
            font.pixelSize: 30
            color: 'white'
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
        }
    }

    Todos {
        id: todoModel
    }

    Component.onCompleted: {
        input.forceActiveFocus()
        todoModel.add("write main.qml")
        todoModel.add("write implementation.rs")
        todoModel.add("check interface.rs")
        todoModel.add("check bindings.cpp")
        todoModel.add("check bindings.h")
        todoModel.add("run rust_qt_binding_generator")
        todoModel.add("write bindings.json")
    }

    Component {
        id: todoDelegate
        Pane {
            height: todoRow.height
            width: parent.width
            RowLayout {
                id: todoRow
                // the active tab determine is this item should be shown
                // 0: all, 1: active, 2: completed
                property bool show: filter.currentIndex === 0
                    || (filter.currentIndex === 1 && !completed)
                    || (filter.currentIndex === 2 && completed)
                visible: show
                width: parent.width
                height: show ? implicitHeight : 0
                CheckBox {
                    id: checkbox
                    checked: completed
                    onToggled: {
                        todoModel.setCompleted(index, checked)
                    }
                }
                Item {
                    id: textline
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    Label {
                        id: label
                        visible: !editInput.visible
                        text: description
                        anchors.fill: parent
                        verticalAlignment: Text.AlignVCenter
                        font.strikeout: completed
                        font.pixelSize: 25
                    }
                    MouseArea {
                        id: mouse
                        anchors.fill: parent
                        hoverEnabled: true
                        onDoubleClicked: {
                            editInput.text = label.text
                            editInput.visible = true
                            editInput.forceActiveFocus()
                        }
                    }
                    Button {
                        text: 'X'
                        visible: mouse.containsMouse && !editInput.visible
                        anchors.right: parent.right
                        onClicked: {
                            todoModel.remove(index)
                        }
                    }
                    TextField {
                        id: editInput
                        visible: false
                        anchors.fill: parent
                        text: description
                        onAccepted: {
                            todoModel.setDescription(index, editInput.text)
                            editInput.visible = false
                        }
                        onActiveFocusChanged: {
                            // hide when focus is lost
                            if (!editInput.activeFocus) {
                                editInput.visible = false
                            }
                        }
                        Keys.onPressed: {
                            // on escape, set value, hide (and lose focus)
                            if (event.key === Qt.Key_Escape) {
                                todoModel.setDescription(index, editInput.text)
                                editInput.visible = false
                                event.accepted = true
                            }
                        }
                    }
                }
            }
        }
    }

    Page {
        anchors.fill: parent
        header: Pane {
            height: row.height
            RowLayout {
                id: row
                width: parent.width
                CheckBox {
                    tristate: true
                    // if there are no todos, do not show this checkbox
                    // but let it take up the same space
                    enabled: todoModel.count > 0
                    opacity: todoModel.count === 0 ? 0 : 1
                    checkState: {
                        if (todoModel.activeCount === 0) {
                            return Qt.Checked
                        } else if (todoModel.activeCount >= todoModel.count) {
                            return Qt.Unchecked
                        }
                        return Qt.PartiallyChecked
                    }
                    onCheckStateChanged: {
                        // if the change is triggered by a user action on this
                        // checkbox, check or uncheck all todos
                        // otherwise, do nothing
                        // (onToggle does not emit for tristate buttons)
                        if (activeFocus) {
                            var checked = checkState !== Qt.Unchecked
                            todoModel.setAll(checked)
                        }
                    }
                }
                TextField {
                    id: input
                    Layout.fillWidth: true
                    placeholderText: qsTr("What needs to be done?")
                    onAccepted: {
                        const todo = text.trim()
                        if (todo) {
                            todoModel.add(todo)
                        }
                        input.clear()
                    }
                }
            }
        }
        Flickable {
            anchors.fill: parent
            ListView {
                anchors.fill: parent
                model: todoModel
                delegate: todoDelegate
            }
        }
        footer: TabBar {
            id: filter
            width: parent.width
            visible: todoModel.count > 0
            TabButton {
                text: qsTr("All")
                checked: true
            }
            TabButton {
                text: qsTr("Active")
            }
            TabButton {
                text: qsTr("Completed")
            }
        }
    }

    footer: Pane {
        visible: todoModel.count > 0
        RowLayout {
            id: footerRow
            width: parent.width
            Layout.fillHeight: true
            Label {
                Layout.fillWidth: true
                text: (todoModel.activeCount === 1)
                    ? qsTr("1 item left")
                    : todoModel.activeCount + qsTr(" items left")
            }
            Button {
                visible: todoModel.count > todoModel.activeCount
                text: qsTr("Clear completed")
                onClicked: todoModel.clearCompleted()
            }
        }
    }
}
